/******************************************************
 *
 * Name:         tsql-left-join-article.sql
 *     
 * Design Phase:
 *     Author:   John Miner
 *     Date:     02-01-2020
 *     Purpose:  T-SQL foundational article.
 * 
 ******************************************************/


--
-- 01 - Create people table w/data
--

-- Drop existing table
DROP TABLE IF EXISTS [Tempdb].[dbo].[PEOPLE]
GO

-- Create new table
CREATE TABLE [Tempdb].[dbo].[PEOPLE]
(
	[PEOPLE_ID] [int] NOT NULL,
	[PEOPLE_FIRST_NAME] [nvarchar](64) NOT NULL,
	[PEOPLE_LAST_NAME] [nvarchar](64) NOT NULL,
	[PEOPLE_ADDRESS_LINE] [nvarchar](128) NOT NULL,
	[PEOPLE_CITY] [nvarchar](64) NOT NULL,
	[PEOPLE_STATE] [nvarchar](32) NOT NULL,
	[PEOPLE_POSTAL_CODE] [nvarchar](16) NOT NULL,
	[PEOPLE_COUNTRY] [nvarchar](64) NOT NULL,
	[START_DATE] [date] NULL,
	[END_DATE] [date] NULL,
	[PERSONS_AGE] AS CASE WHEN START_DATE IS NULL THEN NULL ELSE DATEDIFF(YY, START_DATE, GETDATE()) END
)
GO

-- Add new primary key
ALTER TABLE [Tempdb].[dbo].[PEOPLE]
ADD CONSTRAINT [PK_TBL_PEOPLE] PRIMARY KEY CLUSTERED ([PEOPLE_ID] ASC)
GO

-- Insert sample data
INSERT [Tempdb].[dbo].[PEOPLE] 
(
	[PEOPLE_ID], 
	[PEOPLE_FIRST_NAME], 
	[PEOPLE_LAST_NAME], 
	[PEOPLE_ADDRESS_LINE], 
	[PEOPLE_CITY], 
	[PEOPLE_STATE], 
	[PEOPLE_POSTAL_CODE], 
	[PEOPLE_COUNTRY],
	[START_DATE],
	[END_DATE]
) 
VALUES 
(1, N'Gail', N'Erickson', N'9435 Breck Court', N'Bellevue', N'Washington', N'98004', N'United States','1943-12-10', NULL),
(2, N'Diane', N'Margheim', N'475 Santa Maria', N'Everett', N'Washington', N'98201', N'United States','1949-09-10', NULL),
(3, N'Dylan', N'Miller', N'7048 Laurel', N'Kenmore', N'Washington', N'98028', N'United States','1945-02-18', NULL),
(4, N'Rob', N'Walters', N'5678 Lakeview Blvd.', N'Minneapolis', N'Minnesota', N'55402', N'United States','1946-10-01', NULL),
(5, N'Ken', N'Snchez', N'4350 Minute Dr.', N'Newport Hills', N'Washington', N'98006', N'United States','1936-04-01', '2020-09-15'),
(6, N'Roberto', N'Tamburello', N'2137 Birchwood Dr', N'Redmond', N'Washington', N'98052', N'United States','1950-07-30', NULL),
(7, N'Terri', N'Duffy', N'7559 Worth Ct.', N'Renton', N'Washington', N'98055', N'United States','1947-06-08', NULL);
GO

-- Show the data
SELECT * FROM [Tempdb].[dbo].[PEOPLE];
GO



--
-- 02 - Create pets table w/data
--

-- Drop existing table
DROP TABLE IF EXISTS [Tempdb].[dbo].[PETS]
GO

-- Create new table
CREATE TABLE [Tempdb].[dbo].[PETS]
(
	[PET_ID] [int] NOT NULL,
	[PET_TYPE] [varchar](16) NOT NULL,
	[PET_NAME] [varchar](16) NOT NULL,
	[PET_BREED] [varchar](16) NOT NULL,
	[PET_GENDER] [varchar](16) NOT NULL,
	[PET_AGE] [numeric](2, 1) NOT NULL,
	[START_DATE] [date] NULL,
	[END_DATE] [date] NULL
)

-- Add new primary key
ALTER TABLE [Tempdb].[dbo].[PETS]
ADD CONSTRAINT [PK_TBL_PETS] PRIMARY KEY CLUSTERED ([PET_ID] ASC)
GO


-- Insert sample data
INSERT [Tempdb].[dbo].[PETS] 
(
	[PET_ID], 
	[PET_TYPE], 
	[PET_NAME], 
	[PET_BREED], 
	[PET_GENDER], 
	[PET_AGE],
	[START_DATE],
	[END_DATE]
) 
VALUES 
(1, N'CAT', N'SAMMY', N'ABYSSINIAN', N'MALE', 2.5,'2020-01-01', NULL),
(2, N'DOG', N'HENRY', N'AMERICAN BULLDOG', N'MALE', 1.3, '2020-02-01', NULL),
(3, N'CAT', N'LUCY', N'BOMBAY', N'FEMALE', 4.0, '2020-03-01', NULL),
(4, N'DOG', N'CLEO', N'AMERICAN PITBULL', N'FEMALE', 6.9, '2020-04-01', NULL),
(5, N'CAT', N'BELLA', N'PERSIAN', N'FEMALE', 8.7, '2020-05-01', '2020-09-05'),
(6, N'DOG', N'ZAC', N'BASSET HOUND', N'MALE', 4.0, '2020-06-01', NULL);
GO

-- Show the data
SELECT * FROM [Tempdb].[dbo].[PETS];
GO



--
-- 03 - Create own table w/data
--

-- Drop existing table
DROP TABLE IF EXISTS [Tempdb].[dbo].[OWN]
GO

-- Create new table
CREATE TABLE [Tempdb].[dbo].[OWN]
(
	[PEOPLE_ID] [int] NOT NULL,
	[PET_ID] [int] NOT NULL,
	[START_DATE] [date] NULL,
	[END_DATE] [date] NULL
) ON [PRIMARY]
GO

-- Add foreign key 4 people
ALTER TABLE [Tempdb].[dbo].[OWN] WITH CHECK 
ADD CONSTRAINT [FK_TBL_OWN_PEOPLE] FOREIGN KEY([PEOPLE_ID])
REFERENCES [Tempdb].[dbo].[PEOPLE] ([PEOPLE_ID])
GO

-- Add foreign key 4 pets
ALTER TABLE [Tempdb].[dbo].[OWN] WITH CHECK 
ADD CONSTRAINT [FK_TBL_OWN_PETS] FOREIGN KEY([PET_ID])
REFERENCES [Tempdb].[dbo].[PETS] ([PET_ID])
GO

-- Insert sample data
INSERT INTO [Tempdb].[dbo].[OWN]
(
	[PEOPLE_ID],
	[PET_ID],
	[START_DATE],
	[END_DATE]
)
VALUES
(1, 1, '2020-01-08', NULL),
(2, 2, '2020-02-09', NULL),
(3, 3, '2020-03-03', NULL),
(4, 4, '2020-04-11', NULL),
(5, 5, '2020-05-15', '2020-09-05'),
(6, 6, '2020-06-03', NULL)
GO

-- Show the data
SELECT * FROM [Tempdb].[dbo].[OWN];
GO


--
-- 4 - How many people want pets?
--

SELECT 
	P.PEOPLE_ID,
	P.PEOPLE_FIRST_NAME,
	P.PEOPLE_LAST_NAME,
	P.PERSONS_AGE
FROM 
	[Tempdb].[dbo].[PEOPLE] AS P
LEFT JOIN
    [Tempdb].[dbo].[OWN] AS O
ON
	P.PEOPLE_ID = O.PEOPLE_ID
WHERE 
	O.PEOPLE_ID IS NULL
GO


--
-- 5 - How many people have pets?
--

SELECT 
	P.PEOPLE_ID,
	P.PEOPLE_FIRST_NAME,
	P.PEOPLE_LAST_NAME,
	P.PERSONS_AGE
FROM 
	[Tempdb].[dbo].[PEOPLE] AS P
LEFT JOIN
    [Tempdb].[dbo].[OWN] AS O
ON
	P.PEOPLE_ID = O.PEOPLE_ID
WHERE 
	O.PEOPLE_ID IS NOT NULL
GO


--
-- 6 - Pet owners between 01 Jan & 31 May 2020
--

SELECT 
	P.PEOPLE_ID,
	P.PEOPLE_FIRST_NAME,
	P.PEOPLE_LAST_NAME,
	P.PERSONS_AGE,
	O.*
FROM 
	[Tempdb].[dbo].[PEOPLE] AS P
LEFT JOIN
    [Tempdb].[dbo].[OWN] AS O
ON
	P.PEOPLE_ID = O.PEOPLE_ID AND
	(O.[START_DATE] >= '2020-01-01' AND O.[START_DATE] < '2020-06-01')
WHERE 
	O.PEOPLE_ID IS NOT NULL 
GO


--
-- 7 - Pet owners between 01 Jan & 15 Sep 2020
--

SELECT 
	P.PEOPLE_ID,
	P.PEOPLE_FIRST_NAME,
	P.PEOPLE_LAST_NAME,
	P.PERSONS_AGE
FROM 
	[Tempdb].[dbo].[PEOPLE] AS P
LEFT JOIN
    [Tempdb].[dbo].[OWN] AS O
ON
	P.PEOPLE_ID = O.PEOPLE_ID AND
	(O.[START_DATE] >= '2020-01-01' AND O.[START_DATE] < '2020-09-15')
WHERE 
	O.PEOPLE_ID IS NOT NULL 
GO


--
-- 8 - Any deceased pets or owners
--

SELECT 
	P.PEOPLE_ID AS ID,
	P.PEOPLE_FIRST_NAME + ' ' + P.PEOPLE_LAST_NAME AS NAMES,
	P.PERSONS_AGE AS AGE
FROM 
	[Tempdb].[dbo].[PEOPLE] AS P
WHERE 
	YEAR(P.END_DATE) = 2020
UNION
SELECT 
	P.PET_ID AS ID,
	P.PET_NAME AS NAMES,
	P.PET_AGE AS AGE
FROM
	[Tempdb].[dbo].[PETS] AS P
WHERE
	YEAR(P.END_DATE) = 2020
GO


--
-- 9 - New cat came in October & was assigned.
--

INSERT [Tempdb].[dbo].[PETS] VALUES
(7, N'CAT', N'SHIRLEY', N'RUSSIAN BLUE', N'FEMALE', 1.5, '2020-10-05', NULL);

INSERT INTO [Tempdb].[dbo].[OWN] VALUES
(7, 7, '2020-07-07', NULL);


--
-- 10 - How many pets & people in 2020
--

SELECT 
	'PEOPLE' AS THING,
	COUNT(*) AS TOTAL
FROM 
	[Tempdb].[dbo].[PEOPLE] AS P
LEFT JOIN
    [Tempdb].[dbo].[OWN] AS O
ON
	P.PEOPLE_ID = O.PEOPLE_ID AND
	(YEAR(O.START_DATE) <= 2020) AND
	(O.[END_DATE] IS NULL OR YEAR(O.[END_DATE]) = 2020)

UNION

SELECT 
	P.PET_TYPE AS THING,
	COUNT(*) AS TOTAL
FROM 
	[Tempdb].[dbo].[PETS] AS P
LEFT JOIN
    [Tempdb].[dbo].[OWN] AS O
ON
	P.PET_ID = O.PET_ID AND
	(YEAR(O.START_DATE) <= 2020) AND
	(O.[END_DATE] IS NULL OR YEAR(O.[END_DATE]) = 2020)
GROUP BY
	P.PET_TYPE
GO



--
-- 12 - Theta Join - Prior Pet Adoptions
--

SELECT 
	P.PEOPLE_ID, 
	P.PEOPLE_FIRST_NAME,
	P.PEOPLE_LAST_NAME,
	O.PEOPLE_ID,
	O.PET_ID,
	A.PET_TYPE,
	A.PET_NAME,
	A.PET_GENDER
FROM 
	[Tempdb].[dbo].[PEOPLE] AS P
LEFT JOIN
    [Tempdb].[dbo].[OWN] AS O
ON
	P.PEOPLE_ID > O.PEOPLE_ID 
LEFT JOIN
    [Tempdb].[dbo].[PETS] AS A
ON
	O.PET_ID = A.PET_ID
ORDER BY
	P.PEOPLE_ID, O.PET_ID
GO



--
-- 13 - Remove tables from tempdb
--

-- Drop existing table 3
DROP TABLE IF EXISTS [Tempdb].[dbo].[OWN]
GO

-- Drop existing table 1
DROP TABLE IF EXISTS [Tempdb].[dbo].[PEOPLE]
GO

-- Drop existing table 2
DROP TABLE IF EXISTS [Tempdb].[dbo].[PETS]
GO



--
-- 14 - Three dot notation
--

-- Drop Foreign Key
ALTER TABLE [AdventureWorksDW].[dbo].[FactInternetSales]
DROP CONSTRAINT [FK_FactInternetSales_DimSalesTerritory];
GO

-- Update - random 1% of records
UPDATE F
SET 
  F.SalesTerritoryKey = 0
FROM 
  [AdventureWorksDW].[dbo].[FactInternetSales] AS F
LEFT JOIN
(
  SELECT 
    [SalesOrderNumber], 
    [SalesOrderLineNumber],
    CAST(RAND(CHECKSUM(NEWID())) * 100 as INT) + 1 AS RN
  FROM 
    [AdventureWorksDW].[dbo].[FactInternetSales]
) AS R
ON 
  F.[SalesOrderNumber] = R.[SalesOrderNumber] AND
  F.[SalesOrderLineNumber] = R.[SalesOrderLineNumber] 
WHERE
  R.RN = 1
GO

-- Save keys of unmatched records
SELECT 
    F.[SalesOrderNumber], 
    F.[SalesOrderLineNumber],
    F.[SalesTerritoryKey]
INTO
    [AdventureWorksDW].[bac].[BadRecords]
FROM 
    [AdventureWorksDW].[dbo].[FactInternetSales] AS F
LEFT JOIN
    [AdventureWorksDW].[dbo].[DimSalesTerritory] AS D
ON
    F.SalesTerritoryKey = D.SalesTerritoryKey
WHERE
    D.[SalesTerritoryKey] IS NULL
GO

-- Drop the constraint
ALTER TABLE [dbo].[FactInternetSalesReason] 
DROP CONSTRAINT [FK_FactInternetSalesReason_FactInternetSales]
GO

-- Delete records
DELETE
FROM F
FROM 
    OPENROWSET( 'SQLNCLI', 'Server=vm4sql19;Trusted_Connection=yes;', AdventureWorksDW.dbo.FactInternetSales ) AS F
JOIN
    OPENDATASOURCE('SQLNCLI', 'Server=vm4sql19;Trusted_Connection=yes;').AdventureWorksDW.bac.BadRecords AS B
ON
    F.[SalesOrderNumber] = B.[SalesOrderNumber] AND
    F.[SalesOrderLineNumber] = B.[SalesOrderLineNumber] 
GO



--
-- 15 - four dot notation
--

-- Insert records from cloud to on-premise
INSERT INTO AdventureWorksDW.dbo.FactInternetSales
SELECT
  AF.*
FROM 
  MyAzureDb.dbs4advwrks.dbo.FactInternetSales  AS AF
JOIN
  AdventureWorksDW.bac.BadRecords AS WF
ON 
  AF.[SalesOrderNumber] = WF.[SalesOrderNumber] AND
  AF.[SalesOrderLineNumber] = WF.[SalesOrderLineNumber] 
GO
